package org.nm.challenge.arrays;


/**
 * @author rsqrd
 *
 * Not ThreadSafe
 */
public class Array2D implements IArray2D {

	private final double[][] data;
	private final int size_x;
	private final int size_y;

	/**
	 * Create a new Array2D as a copy of another
	 * 2D array.
	 * 
	 * @param other
	 */
	public Array2D(IArray2D other) {
		size_y = other.getSizeY();
		size_x = other.getSizeX();
		double[][] otherData = other.getData();
		if (otherData.length != size_y) {
			throw new IllegalArgumentException("other array is not in consistent state");
		}
		this.data = new double[size_y][];
		for (int i = 0; i < size_y; ++i) {
			if (size_x != otherData[i].length) {
				throw new IllegalArgumentException("other array is not in consistent state");
			}
			this.data[i] = new double[size_x];
		}
		copy(otherData);
	}

	/**
	 * Create an empty Array2D of size n by m.
	 * 
	 * @param n
	 * @param m
	 */
	public Array2D(int n, int m) {
		this.size_y = n;
		this.size_x = m;
		data = new double[n][m];
	}

	/* (non-Javadoc)
	 * @see org.nm.challenge.arrays.IArray2D#copy(org.nm.challenge.arrays.IArray2D)
	 */
	public void copy(IArray2D other) {
		double[][] otherData = other.getData();
		if (this.data.length != otherData.length) {
			throw new IllegalArgumentException("arrays not same size");
		}
		copy(otherData);
	}

	/* (non-Javadoc)
	 * @see gov.lanl.nmsc.IArray2D#getData()
	 */
	public double[][] getData() {
		return data;
	}

	/* (non-Javadoc)
	 * @see org.nm.challenge.arrays.IArray2D#getSizeX()
	 */
	public int getSizeX() {
		return size_x;
	}

	/* (non-Javadoc)
	 * @see org.nm.challenge.arrays.IArray2D#getSizeY()
	 */
	public int getSizeY() {
		return size_y;
	}

	/* (non-Javadoc)
	 * @see org.nm.challenge.arrays.IArray2D#getTotalSizeY()
	 */
	public int getTotalSizeY() {
		return size_y;
	}

	/**
	 * Copy otherData into this' data.
	 * 
	 * @param otherData
	 */
	private void copy(double[][] otherData) {
		for (int i = 0; i < size_y; ++i) {
			if (this.data[i].length != otherData[i].length) {
				throw new IllegalArgumentException("arrays not same size");
			}
			System.arraycopy(otherData[i], 0, this.data[i], 0, otherData[i].length);
		}
	}

}
