package org.nm.challenge.display;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;

import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleInsets;
import org.nm.challenge.arrays.IArray2D;

public class JFreeChartDisplayPanel extends JPanel implements IDisplay {

	private static final long serialVersionUID = 1L;
	private static final int NDATA = 11;
	private final XYSeriesCollection dataset;
	private final JFreeChart chart;
	/**
	 * Creates a chart.
	 *
	 * @param dataset the data for the chart.
	 *
	 * @return a chart.
	 */
	private static JFreeChart createChart(XYDataset dataset) {
		// create the chart...
		JFreeChart chart = ChartFactory.createXYLineChart(
				"display title", // chart title
				"X", // x axis label
				"mass", // y axis label
				dataset, // data
				PlotOrientation.VERTICAL,
				true, // include legend
				true, // tooltips
				false // urls
		);
		// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
		chart.setBackgroundPaint(Color.white);
		// get a reference to the plot for further customisation...
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setBackgroundPaint(Color.lightGray);
		plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);
		//XYLineAndShapeRenderer renderer	= (XYLineAndShapeRenderer) plot.getRenderer();
		//renderer.setShapesVisible(true);
		//renderer.setShapesFilled(true);
		// change the auto tick unit selection to integer units only...
		NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setAutoRange(false);
		rangeAxis.setRange(0.0, 10.0);
		//rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		// OPTIONAL CUSTOMISATION COMPLETED.
		return chart;
	}	/**
	 * Creates a sample dataset.
	 *
	 * @return a sample dataset.
	 */
	private static XYSeriesCollection createDataset() {
		XYSeriesCollection dataset = new XYSeriesCollection();
		for (int nj = 0; nj < NDATA; ++nj) {
			XYSeries series = new XYSeries("mass " + nj);
			dataset.addSeries(series);
		}
		return dataset;
	}
	
	public JFreeChartDisplayPanel() {
		GridLayout layout = new GridLayout(1,1);
		this.setLayout(layout);
		dataset = createDataset();
		chart = createChart(dataset);
		ChartPanel chartPanel = new ChartPanel(chart);
		//chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
		add(chartPanel);
		chartPanel.setMinimumSize(new Dimension(250, 135));
		chartPanel.setPreferredSize(new Dimension(500, 270));
		chartPanel.setMaximumSize(new Dimension(5000, 270));
	}
	
	public void update(final int iteration, final IArray2D mass) {
		chart.setTitle("iteration: " + iteration);
		dataset.removeAllSeries();
		double[][] data2d = mass.getData();
		int dj = data2d.length / NDATA;
		for (int nj = 0; nj < NDATA; ++nj) {
			int j = (nj - NDATA/2)*dj + data2d.length/2;
			XYSeries series = new XYSeries("mass[" + j + "]");
			double[] data = data2d[j];
			for (int i=0; i <data.length; ++i) {
				//double datum = Math.log(data[i]);
				double datum = data[i];
				series.add(i, datum);
			}
			dataset.addSeries(series);
		}
	}
}
