package org.nm.challenge.wave;

import org.nm.challenge.arrays.IArray2D;
import org.nm.challenge.arrays.SubArray2D;

public class SubWaveProblemSetup implements IProblemSetup {

	private SubArray2D HNew;
	private SubArray2D UNew;
	private SubArray2D VNew;

	private SubArray2D H;
	private SubArray2D U;
	private SubArray2D V;

	private final int size_x;
	private final int size_y;
	private final int numRanks;
	private final int rank;

	private final int offset_y;
	private final int total_size_y;
	private final boolean wavePhysics;

	public SubWaveProblemSetup(int rank, int numRanks, IProblemSetup parentProblemSetup) {
		this.rank = rank;
		this.numRanks = numRanks;
		this.size_x = parentProblemSetup.getSizeX();
		this.size_y = calcSizeY(parentProblemSetup.getSizeY());
		this.offset_y = calcYOffset(parentProblemSetup.getSizeY());
		this.total_size_y = parentProblemSetup.getTotalSizeY();
		this.wavePhysics = parentProblemSetup.isWavePhysics();
		this.H = new SubArray2D(parentProblemSetup.getH(), this.size_y+2,
				this.offset_y);
		this.U = new SubArray2D(parentProblemSetup.getU(), this.size_y+2,
				this.offset_y);
		this.V = new SubArray2D(parentProblemSetup.getV(), this.size_y+2,
				this.offset_y);
		this.HNew = new SubArray2D(parentProblemSetup.getNewH(), this.size_y+2,
				this.offset_y);
		this.UNew = new SubArray2D(parentProblemSetup.getNewU(), this.size_y+2,
				this.offset_y);
		this.VNew = new SubArray2D(parentProblemSetup.getNewV(), this.size_y+2,
				this.offset_y);
	}

	public IArray2D getH() {
		return H;
	}

	public IArray2D getU() {
		return U;
	}

	public IArray2D getV() {
		return V;
	}

	public IArray2D getNewH() {
		return HNew;
	}

	public IArray2D getNewU() {
		return UNew;
	}

	public IArray2D getNewV() {
		return VNew;
	}

	public int getNumRanks() {
		return numRanks;
	}

	public int getOffsetX() {
		return 0;
	}

	public int getOffsetY() {
		return offset_y;
	}

	public int getSizeX() {
		return size_x;
	}

	public int getSizeY() {
		return size_y;
	}

	public int getTotalSizeX() {
		return size_x;
	}

	public int getTotalSizeY() {
		return total_size_y;
	}

	public boolean isWavePhysics() {
		return wavePhysics;
	}

	public void swapArrays() {
		SubArray2D tmp;
		tmp = H;
		H = HNew;
		HNew = tmp;
		tmp = U;
		U = UNew;
		UNew = tmp;
		tmp = V;
		V = VNew;
		VNew = tmp;
	}

	private int calcSizeY(int total_size_y) {
		int size = total_size_y / numRanks + ((rank < (total_size_y % numRanks)) ? 1 : 0);
		return size;
	}

	private int calcYOffset(int total_size_y) {
		int offset = rank * (total_size_y / numRanks);
		if (rank > (total_size_y % numRanks)) {
			offset += (total_size_y % numRanks);
		}
		else {
			offset += rank;
		}
		return offset;
	}
}
